#!/usr/bin/perl -w 
# Program to generate Affected File information

use lib "/usr/share/scmbug/lib";

use strict;
use CGI qw(:standard);
use WebReportUtilities;
use Scmbug::ActivityUtilities;

# Make sure text isn't buffered
$|=1;

my $web_utils = WebReportUtilities->new();
my $webreports_config = $web_utils->webreports_config();

main();
exit;

sub main {
    my $run_report = param("run_report");
    my $bugids = param("bugids");
    my $viewvcPath = param("viewvc");

    if( defined($run_report) && $run_report ne "Y" ) {
        display_form();
    } else {
        run_report($bugids, $viewvcPath);
    }
}


#
# Displays the entry form
#
sub display_form {

    $web_utils->standard_header("Affected Files Report");
    
    print "<BLOCKQUOTE>";
    print "This report will retrieve all of the affected files for a given bug ";
    print "(or list of bugs - comma seperated) and group the file list under ";
    print "changes for each file.  A link to viewvc for each file will be created.";
    print "</BLOCKQUOTE><p>\n";
    
    print "<form name=theForm action=/scmbug-cgi/affected_files.pl>";

    print "    <table border=0 id=table1>";
    print "	    <tr>";
    print "		<td width=267>Repository</td>";
    print "		<td>\n";
    
    print "    <select size=1 name=viewvc>";
    
    foreach my $repository ( @{ $webreports_config->{repositories}} ) {
        print "    <option value=\"" . $repository->{viewvc} . "\">";
	print $repository->{name} . "</option>\n";
    }
    print "			</select></td>";
    print "		</tr>";
    print "	<tr>";
    print "		<td width=267>Bug IDs:</td>";
    print "		<td><input type=text name=bugids size=40></td>";
    print "		</tr>";
    print "		<tr>";
    print "			<td width=267>&nbsp;</td>";
    print "			<td>&nbsp;</td>";
    print "		</tr>";
    print "	</table>";
    print "	<p align=center><input type=submit value=\"Submit\">&nbsp;&nbsp;&nbsp; <input type=reset value=\"Reset\"></p>";
    print "</form>";
    print "</p>";
    print "</body></html>";
}

#
# Runs the report
#
sub run_report {
    my $bugids = shift;
    my $viewvcPath = shift;

    $web_utils->standard_header("Affected Files For " . $bugids);

    my $activityUtils = Scmbug::ActivityUtilities->new( $webreports_config->{daemon_location},
							$webreports_config->{daemon_port} );
    my $results = $activityUtils->get_affected_files( $bugids );

    if( $results ne "" ) {
        print_html_data( sortData($results), $viewvcPath );
    }

    print end_html();
}



#####################
sub print_html_data
{
    my $changeList = shift;
    my $viewvcPath = shift;

    print "<TABLE width=90%>";
    print "<TR><TD><B>Filename</B></TD><TD><B>Revision Changes</B></TD></TR>";

    my $changeset;
    foreach my $changeset ( @{ $changeList } ) {

        print "<TR><TD>";
        print get_html_link_for_file($viewvcPath, $changeset->{file});
        print "</TD><TD>";


	# Now loop the list of revisions
	my $rev;
	my $loop = 0;
	foreach $rev ( @{$changeset->{revisions}}) {
	    if( $rev eq "0" ) {
    	    	print "<I>created</I>";
    	    } elsif( $rev eq "-1" ) {
    	        print "<I>deleted</I>";
    	    } else {
	        print get_html_link_for_file( $viewvcPath, $changeset->{file}, $rev );
    	    }
 
            if( ($loop % 2) != 0 ) {
	        print "&nbsp;&nbsp;";
	    } else {
	        print " --&gt; ";
	    }
	    $loop++;	
	}
    	print "</TD></TR>\n";
    }
    
    print "</TABLE>";


}

#####################
sub get_html_link_for_file
{
    my $viewvcPath = shift;
    my $filename = shift;
    my $revision = shift;
    
    if( !defined($viewvcPath) || $viewvcPath eq "" ) {
        if( !defined($revision) || $revision eq "" || $revision eq "0") {
            return $filename;
        } else {
            return $revision;
        }
    }
    
    my $html_link = "<A HREF=\"";
    $html_link .= $viewvcPath . $filename;

    if( !defined($revision) || $revision eq "" || $revision eq "0") {
        $html_link .= "?view=log\">";
        $html_link .= $filename;
    } else {
        $html_link .= "?view=markup&rev=" . $revision;
        $html_link .= "\">";
        $html_link .= $revision;
    }
        
    $html_link .= "</A>\n";
    
    return $html_link;
}

#
# Prints the data to the screen
#
sub sortData {
    my $changelist = shift;

    my @filenames = ();

    # First extract the filenames so that they can be sorted
    my $changeset;
    foreach $changeset ( @{ $changelist } ) {
    	my $uniqueFile;
    	my $exists = 0;
        foreach $uniqueFile ( @filenames ) {
            if( $changeset->{ file } eq $uniqueFile ) {
            	$exists = 1;
            }
        }
        if($exists != 1) {
            push @filenames, $changeset->{ file };
        }
    }

    my @sortedlist = ();

    my $file;
    foreach $file ( sort(@filenames) ) {
        my @revlist = ();
        
        # Now get all the revision numbers for each file
        my $changeset;
	foreach $changeset ( @{ $changelist } ) {
	    if( $changeset->{ file } eq $file ) {
		push @revlist, $changeset->{old};
		
		# Can't use zero for both create and delete
		if( $changeset->{ new } eq "0" ) {
		    push @revlist, "-1";
		} else {
		    push @revlist, $changeset->{ new };
		}
	    }
	}
	
	# Sort the version numbers numerically
	@revlist = sort { $a <=> $b } @revlist;
	my @reducedlist = ();
	
	# Remove any numbers that occure twice, this means that
	# we have multiple edits one after each other without any other
	# edits interupting it
	my $item1;
	foreach $item1( @revlist ) {
	    my $item2;
	    my $num_matches = 0;
	    foreach $item2( @revlist ) {
	    	if( $item1 eq $item2 ) {
	    	    $num_matches++;
	    	}
	    }
	    if( $num_matches < 2 ) {
	    	push @reducedlist, $item1;
	    }
	}
	
	# Now check if we have a deleted file, if so need
	# to replace the -1 at the start with a 0 at the end
	if( scalar @reducedlist > 0 ) {
	    my $first = shift( @reducedlist );
	    if( $first eq "-1" ) {
	    	push @reducedlist, -1;
	    } else {
	    	unshift( @reducedlist, $first );
	    }
	}
	
	my $newdata;
	$newdata->{ file } = $file;
	$newdata->{ revisions } = \@reducedlist;
	push @sortedlist, $newdata;
    }

    return \@sortedlist;
}
